﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace MultiThreading
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        BackgroundWorker worker;
        ProgressDialog pd;

        public Window1()
        {
            InitializeComponent();
        }

        #region Update UI by Reporting Progress

        private void btnReportProgress_Click(object sender, RoutedEventArgs e)
        {
            int maxRecords = 1000;

            pd = new ProgressDialog();
            pd.Cancel += CancelProcess;

            System.Windows.Threading.Dispatcher pdDispatcher = pd.Dispatcher;

            worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.WorkerSupportsCancellation = true;

            worker.DoWork += delegate(object s, DoWorkEventArgs args)
            {
                for (int x = 1; x < maxRecords; x++)
                {
                    if (worker.CancellationPending)
                    {
                        args.Cancel = true;
                        return;
                    }

                    System.Threading.Thread.Sleep(10);
                    worker.ReportProgress(Convert.ToInt32(((decimal)x / (decimal)maxRecords) * 100));
                }
            };

            worker.ProgressChanged += delegate(object s, ProgressChangedEventArgs args)
            {
                pd.ProgressText = args.ProgressPercentage.ToString() + "%";
            };

            worker.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs args)
            {
                pd.Close();
            };

            worker.RunWorkerAsync();
            pd.ShowDialog();
        }

        #endregion

        #region Update UI with Dispatcher

        private void btnDispacther_Click(object sender, RoutedEventArgs e)
        {
            int maxRecords = 1000;

            pd = new ProgressDialog();
            pd.Cancel += CancelProcess;

            System.Windows.Threading.Dispatcher pdDispatcher = pd.Dispatcher;

            worker = new BackgroundWorker();
            worker.WorkerSupportsCancellation = true;

            worker.DoWork += delegate(object s, DoWorkEventArgs args)
            {
                for (int x = 1; x < maxRecords; x++)
                {
                    if (worker.CancellationPending)
                    {
                        args.Cancel = true;
                        return;
                    }

                    System.Threading.Thread.Sleep(10);

                    UpdateProgressDelegate update = new UpdateProgressDelegate(UpdateProgressText);
                    pdDispatcher.BeginInvoke(update, Convert.ToInt32(((decimal)x / (decimal)maxRecords) * 100), maxRecords);
                }
            };

            worker.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs args)
            {
                pd.Close();
            };

            worker.RunWorkerAsync();
            pd.ShowDialog();
        }

        public delegate void UpdateProgressDelegate(int percentage, int recordCount);

        public void UpdateProgressText(int percentage, int recordCount)
        {
            pd.ProgressText = string.Format("{0}% of {1} Records", percentage.ToString(), recordCount);
            pd.ProgressValue = percentage;
        }

        #endregion

        void CancelProcess(object sender, EventArgs e)
        {
            worker.CancelAsync();
        }
    }
}
